/*
 * Copyright (c) 2016, Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 * All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef _EMWIN_SUPPORT_H_
#define _EMWIN_SUPPORT_H_

/* Macros for the LCD controller. */
#define APP_ELCDIF LCDIF

#define APP_IMG_HEIGHT 480
#define APP_IMG_WIDTH 800
#define APP_HSW 46			/*!< HSYNC pulse width. */
#define APP_HFP 210			/*!< Horizontal front porch. */
#define APP_HBP 46			/*!< Horizontal back porch. */
#define APP_VSW 23			/*!< VSYNC pulse width. */
#define APP_VFP 22			/*!< Vertical front porch. */
#define APP_VBP 23			/*!< Vertical back porch. */
#define APP_POL_FLAGS \
    (kELCDIF_DataEnableActiveHigh | kELCDIF_VsyncActiveLow | kELCDIF_HsyncActiveLow | kELCDIF_DriveDataOnRisingClkEdge)

/* Display. */
#define LCD_DISP_GPIO GPIO1
#define LCD_DISP_GPIO_PIN 24


#define BOARD_LCD_READABLE 1

#define LCD_WIDTH 800
#define LCD_HEIGHT 480
#define LCD_BITS_PER_PIXEL 32
#define LCD_BYTES_PER_PIXEL (LCD_BITS_PER_PIXEL / 8)

/* Color depth dependent definitions */
#if LCD_BITS_PER_PIXEL == 8
#define DISPLAY_DRIVER GUIDRV_LIN_8
#define COLOR_CONVERSION GUICC_0
#define ELCDIF_PIXEL_FORMAT kELCDIF_PixelFormatRAW8
#define APP_LCDIF_DATA_BUS kELCDIF_DataBus8Bit
#elif LCD_BITS_PER_PIXEL == 16
#define DISPLAY_DRIVER GUIDRV_LIN_16
#define COLOR_CONVERSION GUICC_M565
#define ELCDIF_PIXEL_FORMAT kELCDIF_PixelFormatRGB565
#define APP_LCDIF_DATA_BUS kELCDIF_DataBus16Bit
#else
#define DISPLAY_DRIVER GUIDRV_LIN_OXY_32
#define COLOR_CONVERSION GUICC_M888
#define ELCDIF_PIXEL_FORMAT kELCDIF_PixelFormatXRGB8888
#define APP_LCDIF_DATA_BUS kELCDIF_DataBus16Bit
#endif

/* Define scale factors */
#define GUI_SCALE_FACTOR 1
#define GUI_SCALE_FACTOR_X 1
#define GUI_SCALE_FACTOR_Y 1

/* Use larger fonts */
#define GUI_NORMAL_FONT (&GUI_Font24_ASCII)
#define GUI_LARGE_FONT (&GUI_Font32B_ASCII)

#define GUI_BUFFERS 2
#define GUI_NUMBYTES 200000U /*! Amount of memory assigned to the emWin library */

#define FRAME_BUFFER_ALIGN 64

#define VRAM_SIZE (LCD_HEIGHT * LCD_WIDTH * LCD_BYTES_PER_PIXEL)

#endif
